@@ -9,7 +9,6 @@
 import re
 import os
 import errno
-import signal
 import sys
 import time
 try:
@@ -21,11 +20,14 @@ from paste.deploy import loadapp, loadserver
 import threading
 import atexit
 import logging
-import logging.config
 import ConfigParser
 
 MAXFD = 1024
 
+class DaemonizeException(Exception):
+    pass
+
+
 class ServeCommand(Command):
 
     min_args = 0
@@ -59,10 +61,11 @@ class ServeCommand(Command):
                       dest='server_name',
                       metavar='SECTION_NAME',
                       help="Use the named server as defined in the configuration file (default: main)")
-    parser.add_option('--daemon',
-                      dest="daemon",
-                      action="store_true",
-                      help="Run in daemon (background) mode")
+    if hasattr(os, 'fork'):
+        parser.add_option('--daemon',
+                          dest="daemon",
+                          action="store_true",
+                          help="Run in daemon (background) mode")
     parser.add_option('--pid-file',
                       dest='pid_file',
                       metavar='FILENAME',
@@ -90,7 +93,7 @@ class ServeCommand(Command):
 
 
     if hasattr(os, 'setuid'):
-        # I don't think these are availble on Windows
+        # I don't think these are available on Windows
         parser.add_option('--user',
                           dest='set_user',
                           metavar="USERNAME",
@@ -157,10 +160,6 @@ class ServeCommand(Command):
             else:
                 return self.restart_with_reloader()
 
-        if (self.options.monitor_restart
-            and not os.environ.get(self._monitor_environ_key)):
-            return self.restart_with_monitor()
-
         if cmd not in (None, 'start', 'stop', 'restart', 'status'):
             raise BadCommand(
                 'Error: must give start|stop|restart (not %s)' % cmd)
@@ -192,6 +191,12 @@ class ServeCommand(Command):
             server_spec = app_spec
         base = os.getcwd()
 
+        if getattr(self.options, 'daemon', False):
+            if not self.options.pid_file:
+                self.options.pid_file = 'paster.pid'
+            if not self.options.log_file:
+                self.options.log_file = 'paster.log'
+
         # Ensure the log file is writeable
         if self.options.log_file:
             try:
@@ -201,8 +206,26 @@ class ServeCommand(Command):
                 raise BadCommand(msg)
             writeable_log_file.close()
 
-        if self.options.daemon:
-            self.daemonize()
+        # Ensure the pid file is writeable
+        if self.options.pid_file:
+            try:
+                writeable_pid_file = open(self.options.pid_file, 'a')
+            except IOError, ioe:
+                msg = 'Error: Unable to write to pid file: %s' % ioe
+                raise BadCommand(msg)
+            writeable_pid_file.close()
+
+        if getattr(self.options, 'daemon', False):
+            try:
+                self.daemonize()
+            except DaemonizeException, ex:
+                if self.verbose > 0:
+                    print str(ex)
+                return
+
+        if (self.options.monitor_restart
+            and not os.environ.get(self._monitor_environ_key)):
+            return self.restart_with_monitor()
 
         if self.options.pid_file:
             self.record_pid(self.options.pid_file)
@@ -213,10 +236,6 @@ class ServeCommand(Command):
             sys.stderr = stdout_log
             logging.basicConfig(stream=stdout_log)
 
-        server = self.loadserver(server_spec, name=server_name,
-                                 relative_to=base, global_conf=vars)
-        app = self.loadapp(app_spec, name=app_name,
-                           relative_to=base, global_conf=vars)
         log_fn = app_spec
         if log_fn.startswith('config:'):
             log_fn = app_spec[len('config:'):]
@@ -224,13 +243,19 @@ class ServeCommand(Command):
             log_fn = None
         if log_fn:
             log_fn = os.path.join(base, log_fn)
-            parser = ConfigParser.ConfigParser()
-            parser.read([log_fn])
-            if parser.has_section('loggers'):
-                logging.config.fileConfig(log_fn)
+            self.logging_file_config(log_fn)
+
+        server = self.loadserver(server_spec, name=server_name,
+                                 relative_to=base, global_conf=vars)
+        app = self.loadapp(app_spec, name=app_name,
+                           relative_to=base, global_conf=vars)
 
         if self.verbose > 0:
-            print 'Starting server in PID %i.' % os.getpid()
+            if hasattr(os, 'getpid'):
+                msg = 'Starting server in PID %i.' % os.getpid()
+            else:
+                msg = 'Starting server.'
+            print msg
         try:
             server(app)
         except (SystemExit, KeyboardInterrupt), e:
@@ -253,6 +278,12 @@ class ServeCommand(Command):
                 **kw)
 
     def daemonize(self):
+        pid = live_pidfile(self.options.pid_file)
+        if pid:
+            raise DaemonizeException(
+                "Daemon is already running (PID: %s from PID file %s)"
+                % (pid, self.options.pid_file))
+
         if self.verbose > 0:
             print 'Entering daemon mode'
         pid = os.fork()
@@ -270,7 +301,7 @@ class ServeCommand(Command):
 
         # @@: Should we set the umask and cwd now?
 
-        import resource		# Resource usage information.
+        import resource  # Resource usage information.
         maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
         if (maxfd == resource.RLIM_INFINITY):
             maxfd = MAXFD
@@ -278,23 +309,18 @@ class ServeCommand(Command):
         for fd in range(0, maxfd):
             try:
                 os.close(fd)
-            except OSError:	# ERROR, fd wasn't open to begin with (ignored)
+            except OSError:  # ERROR, fd wasn't open to begin with (ignored)
                 pass
 
         if (hasattr(os, "devnull")):
             REDIRECT_TO = os.devnull
         else:
             REDIRECT_TO = "/dev/null"
-        os.open(REDIRECT_TO, os.O_RDWR)	# standard input (0)
+        os.open(REDIRECT_TO, os.O_RDWR)  # standard input (0)
         # Duplicate standard input to standard output and standard error.
-        os.dup2(0, 1)			# standard output (1)
-        os.dup2(0, 2)			# standard error (2)
+        os.dup2(0, 1)  # standard output (1)
+        os.dup2(0, 2)  # standard error (2)
         
-        if not self.options.pid_file:
-            self.options.pid_file = 'paster.pid'
-        if not self.options.log_file:
-            self.options.log_file = 'paster.log'
-
     def record_pid(self, pid_file):
         pid = os.getpid()
         if self.verbose > 1:
@@ -325,6 +351,7 @@ class ServeCommand(Command):
         for j in range(10):
             if not live_pidfile(pid_file):
                 break
+            import signal
             os.kill(pid, signal.SIGTERM)
             time.sleep(1)
         else:
